using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Widgets;

namespace Escape;

/// @author annilehtinen
/// @version 28.03.2025 / 2
/// <summary>
/// Peli, jossa pelaajaan tavoitteena on vältellä vastustajaa (Barkov) ja kerätä kiekkoja saadakseen pisteitä.
/// </summary>
public class Escape : PhysicsGame
{
    public override void Begin()
    {
        Surfaces borders = Level.CreateBorders(0.5, true, Color.White);
        Camera.ZoomToLevel();
        
        PhysicsObject player = new PhysicsObject(40, 40);
        player.Image = LoadImage("player.png");
        player.CanRotate = false;
        player.LinearDamping = 0.95;
        
        this.Add(player);
        
        SetControls();
        
        PhysicsObject barkov = new PhysicsObject(50, 50);
        barkov.Image = LoadImage("Barkov.png");
        barkov.X = 60;
        barkov.Y = -60;
        this.Add(barkov);

        SpawnPuck();
        Timer.CreateAndStart(4.0, SpawnPuck);
        
        /// <summary>
        /// Peliohjainten asettaminen
        /// </summary>
        void SetControls()
        {
            Keyboard.Listen(Key.Up, ButtonState.Down, MovePlayer, "Liikuta pelaajaa eteenpäin");
            Keyboard.Listen(Key.Left, ButtonState.Down, TurnPlayer, "Käännä pelaajaa vasemmalle", 2.0);
            Keyboard.Listen(Key.Left, ButtonState.Released, TurnPlayer, null, 0.0);
            Keyboard.Listen(Key.Right, ButtonState.Down, TurnPlayer, "Käännä pelaajaa oikealle", -2.0);
            Keyboard.Listen(Key.Right, ButtonState.Released, TurnPlayer, null, 0.0);
        
            Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
        }
       
        /// <summary>
        /// Pelaajan liikuttaminen eteenpäin (myöhemmin animaation kanssa)
        /// </summary>
        void MovePlayer()
        {
            int playerSpeed = 10;
            Vector playerDirection = Vector.FromLengthAndAngle(50.0, player.Angle);
            player.Push(playerDirection * playerSpeed);
        }
        
        /// <summary>
        /// Pelaajan kääntäminen nuolinäppäinten avulla
        /// </summary>
        void TurnPlayer(double angle)
        {
            player.Angle += Angle.FromDegrees(angle);
        }

        void SpawnPuck()
        {
            GameObject puck = new GameObject(18, 18);
            puck.Shape = Shape.Circle;
            puck.Color = Color.Black;
            puck.LifetimeLeft = TimeSpan.FromSeconds(5.0);
            puck.Position = Level.GetRandomPosition();
            this.Add(puck);
        }
    }
}